=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2005-09-29 20:34:25 +0000
+++ bzrlib/builtins.py	2005-09-30 06:39:42 +0000
@@ -936,13 +936,20 @@
                 raise BzrError('bzr export --revision takes exactly 1 argument')
             rev_id = revision[0].in_history(b).rev_id
         t = b.revision_tree(rev_id)
-        root, ext = os.path.splitext(dest)
+        arg_root, ext = os.path.splitext(os.path.basename(dest))
+        if ext in ('.gz', '.bz2'):
+            new_root, new_ext = os.path.splitext(arg_root)
+            if new_ext == '.tar':
+                arg_root = new_root
+                ext = new_ext + ext
+        if root is None:
+            root = arg_root
         if not format:
             if ext in (".tar",):
                 format = "tar"
-            elif ext in (".gz", ".tgz"):
+            elif ext in (".tar.gz", ".tgz"):
                 format = "tgz"
-            elif ext in (".bz2", ".tbz2"):
+            elif ext in (".tar.bz2", ".tbz2"):
                 format = "tbz2"
             else:
                 format = "dir"

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2005-09-29 21:03:18 +0000
+++ bzrlib/commit.py	2005-09-30 06:39:42 +0000
@@ -342,7 +342,8 @@
                 deleted_ids.append(ie.file_id)
         if deleted_ids:
             for file_id in deleted_ids:
-                del self.work_inv[file_id]
+                if file_id in self.work_inv:
+                    del self.work_inv[file_id]
             self.branch._write_inventory(self.work_inv)
 
 

=== modified file 'bzrlib/merge.py'
--- bzrlib/merge.py	2005-09-30 03:38:47 +0000
+++ bzrlib/merge.py	2005-09-30 06:39:42 +0000
@@ -24,7 +24,7 @@
 import bzrlib.revision
 from bzrlib.merge_core import merge_flex, ApplyMerge3, BackupBeforeChange
 from bzrlib.changeset import generate_changeset, ExceptionConflictHandler
-from bzrlib.changeset import Inventory, Diff3Merge
+from bzrlib.changeset import Inventory, Diff3Merge, ReplaceContents
 from bzrlib.branch import Branch
 from bzrlib.errors import BzrCommandError, UnrelatedBranches, NoCommonAncestor
 from bzrlib.errors import NoCommits
@@ -53,10 +53,13 @@
     conflict that are not explicitly handled cause an exception and
     terminate the merge.
     """
-    def __init__(self, ignore_zero=False):
+    def __init__(self, this_tree, base_tree, other_tree, ignore_zero=False):
         ExceptionConflictHandler.__init__(self)
         self.conflicts = 0
         self.ignore_zero = ignore_zero
+        self.this_tree = this_tree
+        self.base_tree = base_tree
+        self.other_tree = other_tree
 
     def copy(self, source, dest):
         """Copy the text and mode of a file
@@ -134,6 +137,52 @@
             % filename)
         return "skip"
 
+    def rem_contents_conflict(self, filename, this_contents, base_contents):
+        base_contents(filename+".BASE", self, False)
+        this_contents(filename+".THIS", self, False)
+        self.conflict("Other branch deleted locally modified file %s" %
+                      filename)
+        return ReplaceContents(this_contents, None)
+
+    def abs_this_path(self, file_id):
+        """Return the absolute path for a file_id in the this tree."""
+        relpath = self.this_tree.id2path(file_id)
+        return self.this_tree.tree.abspath(relpath)
+
+    def add_missing_parents(self, file_id, tree):
+        """If some of the parents for file_id are missing, add them."""
+        entry = tree.tree.inventory[file_id]
+        if entry.parent_id not in self.this_tree:
+            return self.create_all_missing(entry.parent_id, tree)
+        else:
+            return self.abs_this_path(entry.parent_id)
+
+    def create_all_missing(self, file_id, tree):
+        """Add contents for a file_id and all its parents to a tree."""
+        entry = tree.tree.inventory[file_id]
+        if entry.parent_id is not None and entry.parent_id not in self.this_tree:
+            abspath = self.create_all_missing(entry.parent_id, tree)
+        else:
+            abspath = self.abs_this_path(entry.parent_id)
+        entry_path = os.path.join(abspath, entry.name)
+        if not os.path.isdir(entry_path):
+            self.create(file_id, entry_path, tree)
+        return entry_path
+
+    def create(self, file_id, path, tree, reverse=False):
+        """Uses tree data to create a filesystem object for the file_id"""
+        from merge_core import get_id_contents
+        get_id_contents(file_id, tree)(path, self, reverse)
+
+    def missing_for_merge(self, file_id, other_path):
+        """The file_id doesn't exist in THIS, but does in OTHER and BASE"""
+        self.conflict("Other branch modified locally deleted file %s" %
+                      other_path)
+        parent_dir = self.add_missing_parents(file_id, self.other_tree)
+        stem = os.path.join(parent_dir, os.path.basename(other_path))
+        self.create(file_id, stem+".OTHER", self.other_tree)
+        self.create(file_id, stem+".BASE", self.base_tree)
+
     def finalize(self):
         if not self.ignore_zero:
             print "%d conflicts encountered.\n" % self.conflicts
@@ -370,7 +419,8 @@
 
     inv_changes = merge_flex(this_tree, base_tree, other_tree,
                              generate_cset_optimized, get_inventory,
-                             MergeConflictHandler(ignore_zero=ignore_zero),
+                             MergeConflictHandler(this_tree, base_tree,
+                             other_tree, ignore_zero=ignore_zero),
                              merge_factory=merge_factory, 
                              interesting_ids=interesting_ids)
 

=== modified file 'bzrlib/merge_core.py'
--- bzrlib/merge_core.py	2005-09-23 09:25:16 +0000
+++ bzrlib/merge_core.py	2005-09-30 06:39:42 +0000
@@ -198,11 +198,14 @@
 
 
 def get_contents(entry, tree):
+    return get_id_contents(entry.id, tree)
+
+def get_id_contents(file_id, tree):
     """Get a contents change element suitable for use with ReplaceContents
     """
-    tree_entry = tree.tree.inventory[entry.id]
+    tree_entry = tree.tree.inventory[file_id]
     if tree_entry.kind == "file":
-        return changeset.FileCreate(tree.get_file(entry.id).read())
+        return changeset.FileCreate(tree.get_file(file_id).read())
     else:
         assert tree_entry.kind in ("root_directory", "directory")
         return changeset.dir_create
@@ -224,7 +227,11 @@
         if contents.old_contents is None and contents.new_contents is None:
             return None
         if contents.new_contents is None:
+            this_contents = get_contents(entry, this)
             if this_path is not None and os.path.exists(this_path):
+                if this_contents != contents.old_contents:
+                    return conflict_handler.rem_contents_conflict(this_path, 
+                        this_contents, contents.old_contents)
                 return contents
             else:
                 return None

=== modified file 'bzrlib/selftest/blackbox.py'
--- bzrlib/selftest/blackbox.py	2005-09-29 20:34:25 +0000
+++ bzrlib/selftest/blackbox.py	2005-09-30 06:39:42 +0000
@@ -207,6 +207,42 @@
         test.runbzr('add goodbye')
         test.runbzr('commit -m setup goodbye')
 
+    def test_export(self):
+        os.mkdir('branch')
+        os.chdir('branch')
+        self.example_branch()
+        self.runbzr('export ../latest')
+        self.assertEqual(file('../latest/goodbye', 'rt').read(), 'baz')
+        self.runbzr('export ../first -r 1')
+        assert not os.path.exists('../first/goodbye')
+        self.assertEqual(file('../first/hello', 'rt').read(), 'foo')
+        self.runbzr('export ../first.gz -r 1')
+        self.assertEqual(file('../first.gz/hello', 'rt').read(), 'foo')
+        self.runbzr('export ../first.bz2 -r 1')
+        self.assertEqual(file('../first.bz2/hello', 'rt').read(), 'foo')
+        self.runbzr('export ../first.tar -r 1')
+        assert os.path.isfile('../first.tar')
+        from tarfile import TarFile
+        tf = TarFile('../first.tar')
+        assert 'first/hello' in tf.getnames(), tf.getnames()
+        self.assertEqual(tf.extractfile('first/hello').read(), 'foo')
+        self.runbzr('export ../first.tar.gz -r 1')
+        assert os.path.isfile('../first.tar.gz')
+        self.runbzr('export ../first.tbz2 -r 1')
+        assert os.path.isfile('../first.tbz2')
+        self.runbzr('export ../first.tar.bz2 -r 1')
+        assert os.path.isfile('../first.tar.bz2')
+        self.runbzr('export ../first.tar.tbz2 -r 1')
+        assert os.path.isfile('../first.tar.tbz2')
+        from bz2 import BZ2File
+        tf = TarFile('../first.tar.tbz2', 
+                     fileobj=BZ2File('../first.tar.tbz2', 'r'))
+        assert 'first.tar/hello' in tf.getnames(), tf.getnames()
+        self.assertEqual(tf.extractfile('first.tar/hello').read(), 'foo')
+        self.runbzr('export ../first2.tar -r 1 --root pizza')
+        tf = TarFile('../first2.tar')
+        assert 'pizza/hello' in tf.getnames(), tf.getnames()
+
     def test_diff(self):
         self.example_branch()
         file('hello', 'wt').write('hello world!')
@@ -261,6 +297,40 @@
         #        assert a.pending_merges() == [b.last_revision()], "Assertion %s %s" \
         #        % (a.pending_merges(), b.last_revision())
 
+    def test_merge_with_missing_file(self):
+        """Merge handles missing file conflicts"""
+        os.mkdir('a')
+        os.chdir('a')
+        os.mkdir('sub')
+        print >> file('sub/a.txt', 'wb'), "hello"
+        print >> file('b.txt', 'wb'), "hello"
+        print >> file('sub/c.txt', 'wb'), "hello"
+        self.runbzr('init')
+        self.runbzr('add')
+        self.runbzr(('commit', '-m', 'added a'))
+        self.runbzr('branch . ../b')
+        print >> file('sub/a.txt', 'ab'), "there"
+        print >> file('b.txt', 'ab'), "there"
+        print >> file('sub/c.txt', 'ab'), "there"
+        self.runbzr(('commit', '-m', 'Added there'))
+        os.unlink('sub/a.txt')
+        os.unlink('sub/c.txt')
+        os.rmdir('sub')
+        os.unlink('b.txt')
+        self.runbzr(('commit', '-m', 'Removed a.txt'))
+        os.chdir('../b')
+        print >> file('sub/a.txt', 'ab'), "something"
+        print >> file('b.txt', 'ab'), "something"
+        print >> file('sub/c.txt', 'ab'), "something"
+        self.runbzr(('commit', '-m', 'Modified a.txt'))
+        self.runbzr('merge ../a/')
+        assert os.path.exists('sub/a.txt.THIS')
+        assert os.path.exists('sub/a.txt.BASE')
+        os.chdir('../a')
+        self.runbzr('merge ../b/')
+        assert os.path.exists('sub/a.txt.OTHER')
+        assert os.path.exists('sub/a.txt.BASE')
+
     def test_pull(self):
         """Pull changes from one branch to another."""
         os.mkdir('a')

